<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 */

// Import the iMarkaPageInfosProvider interface.
Yii::import('common.modules.marketingautomation.components.iMarkaPageInfosProvider');

/**
 * Implementation of iMarkaPageInfosProvider, which provides page infos from the Gentics
 * ContentRepository. This provider can be used to handle two types of MarkA page IDs:
 * 1. CMS page IDs without a prefix (e.g. '4711'), as the defaultPageInfosProvider
 * 2. CMS page IDs with a 'c' prefix (e.g. 'c2.4711') for pages in a channel
 *
 * This class is used by default as the default iMarkaPageInfosProvider and if used as such,
 * no provider for the 'c' prefix needs to be configured in the config explicitly.
 */
 class ContentRepositoryMarkaPageInfosProvider implements iMarkaPageInfosProvider
 {

    /**
     * Retrieves the requested attributes for the specified Marketing Automation page IDs,
     * which have the prefix that this iMarkaPageInfosProvider handles,
     * and returns an associative array with all page infos found in the repository. Each 
     * Marketing Automation page ID from the markaPageIds array will be associated with one
     * page infos associative array. Each of these page infos arrays contains the requested page attributes
     * (e.g. array('name' => 'Soccer', 'url' => '/sports/Soccer.html') for the
     * corresponding page. If any of the attributes was not found for a page or was empty, the
     * entry in the array will exist, but will be NULL. If no attributes were found for a
     * particular page, there will be no entry for that page ID in the returned array.
     *
     * Note that this function does not interact with the cache in any way. The SuggestionsController
     * will first check if the page infos for a page ID are available in the cache and only if they are not,
     * add it to the array, which is passed to this function. The insertion of page infos into the cache
     * is also handled by the SuggestionsController.
     * 
     * @param array $markaPageIds the array of page IDs with this iMarkaPageInfosProvider's
     * prefix, as returned by Marketing Automation
     * @param array $pageAttributes the page attributes (CMS or repository attributes,
     * not Marketing Automation attributes) that should be retrieved for each page
     * @return array
     */
    public function getPageInfos($markaPageIds, $pageAttributes)
    {
        $parsedPageIds = self::_parsePageIds($markaPageIds);
        $pageIdsGroupedByChannel = self::_groupPageIdsByChannel($parsedPageIds);
        $finalPageInfos = array();
        foreach ($pageIdsGroupedByChannel as $channelId => $group) {
            $query = array('filter' => self::_createFilterString($group), 'type' => 'php');
            if ($channelId !== 'master') {
                $query['channel_id'] = $channelId;
            }
            $response = YII::app()->repositoryApi->request($query, $pageAttributes);
            $rawPageInfos = self::_prepareResponse($response);
            if ($rawPageInfos === false) {
                continue;
            }
            $this->_createSuggestedPageInfos($rawPageInfos, $channelId, $parsedPageIds, $finalPageInfos, $pageAttributes);
        }
        return $finalPageInfos;
    }

    /**
     * Parses the specified array of Marketing Automation page IDs and returns
     * an array of parsed page IDs. Each of these is an object, which contains
     * the following properties: markaPageId (the Marketing Automation page ID),
     * channelId (the channel ID of the page, if present, or NULL otherwise),
     * and pageId (the page ID used by the CMS, this is already prefixed with '10007.').
     * Each array entry has the key 'c' . channelId . '.' . CMSPageId or
     * only CMSPageId (if it has no channelId).
     * @param array $markaPageIds the array of page IDs passed to getPageInfos()
     * @return array
     */
    private static function _parsePageIds(&$markaPageIds)
    {
        $ret = array();
        $size = count($markaPageIds);
        for ($i = 0; $i < $size; ++$i) {
            $currPageId = $markaPageIds[$i];
            $currPage = new stdClass();
            $currPage->markaPageId = $currPageId;
            if (substr($currPageId, 0, 1) === 'c') {
                $separatorIndex = strpos($currPageId, '.', 2);
                if ($separatorIndex === false || $separatorIndex === strlen($currPageId) - 1) {
                    die('Illegal Marketing Automation Page ID returned from tracker: ' . $currPageId);
                }
                $currPage->channelId = substr($currPageId, 1, $separatorIndex - 1);
                $currPage->pageId = '10007.' . substr($currPageId, $separatorIndex + 1);
            } else {
                $currPage->channelId = NULL;
                $currPage->pageId = '10007.' . $currPageId;
            }
            $key = $currPage->pageId;
            if (!empty($currPage->channelId)) {
                $key = 'c' . $currPage->channelId . '.' . $key;
            }
            $ret[$key] = $currPage;
        }
        return $ret;
    }

    /**
     * Groups the specified parsed page IDs, which do not have their cachedPageInfo
     * property set by their channel ID.
     * Returns an array, which contains an entry for each channelId. Each of 
     * these entries is an array of all the parsed page IDs, which belong
     * to that channel. Page IDs, which have NULL as their channel ID are 
     * placed in the entry with key 'master'. Page IDs, which have their
     * cachedPageInfo property set are not included.
     * 
     * @param array $parsedPageIds the array of parsed page IDs
     * @return array
     */
    private static function _groupPageIdsByChannel(&$parsedPageIds)
    {
        $ret = array();
        foreach ($parsedPageIds as $key => $pageId) {
            if (!empty($pageId->cachedPageInfo)) {
                continue;
            }
            $channelId = $pageId->channelId;
            if ($channelId === NULL) {
                $channelId = 'master';
            }
            if (!array_key_exists($channelId, $ret)) {
                $ret[$channelId] = array();
            }
            array_push($ret[$channelId], $pageId);
        }
        return $ret;
    }
    
    /**
     * Creates and returns a filter string with the specified parsed page IDs
     * for making a content repository request.
     * 
     * @param array $parsedPageIds the array of parsed page IDs
     * @return string
     */
    private static function _createFilterString(&$parsedPageIds)
    {
        $ret = 'object.contentid CONTAINSONEOF [';
        $first = true;
        foreach ($parsedPageIds as $pageId) {
            if (!$first) {
                $ret .= ',';
            } else {
                $first = false;
            }
            $ret .= '"' . $pageId->pageId . '"';
        }
        $ret .= ']';
        return $ret;
    }

    /**
     * Prepare API response. Convert response to useful form.
     * This method is based on RepositoryApi::_prepareResponse()
     *
     * @param array $response prepared response
     *
     * @return array|bool prepared object.
     *
     */
    private static function _prepareResponse($response)
    {
        $responseData = unserialize($response);
        if (@$responseData['status'] == 'ok') {
            unset($responseData['status']);
            if (count($responseData) > 0) {
                return $responseData;
            }
            return false;
        } else {
            Yii::log(print_r($responseData, true), CLogger::LEVEL_TRACE);
        }
        return false;
    }

    /**
     * Creates suggested page info associative arrays for all specified raw page infos
     * and inserts each of them into the finalPageInfos array with the MarkA page ID as the key.
     * 
     * @param array $rawPageInfos the raw page infos
     * @param string $channelId the channel ID of all the pages in rawPageInfos
     * @param array $parsedPageIds the parsedPageIds array (used to obtain the markaPageId)
     * @param array $finalPageInfos the associative array, into which the page infos should be inserted 
     * (with the MarkA page ID as the key)
     * @param array $pageAttributes the page attributes that were supposed to be retrieved for each page
     */
    private function _createSuggestedPageInfos(&$rawPageInfos, &$channelId, &$parsedPageIds, &$finalPageInfos, &$pageAttributes)
    {
        if ($channelId !== 'master') {
            $channelPrefix = 'c' . $channelId . '.';
        } else {
            $channelPrefix = '';
        }
        foreach ($rawPageInfos as $pageId => $rawPageInfo) {
            $suggestedPageInfo = array();
            foreach ($pageAttributes as $attribute) {
                if (array_key_exists($attribute, $rawPageInfo['attributes'])) {
                    $suggestedPageInfo[$attribute] = $rawPageInfo['attributes'][$attribute];
                } else {
                    $suggestedPageInfo[$attribute] = NULL;
                }
            }
            if (!empty($suggestedPageInfo)) {
                $markaPageId = $parsedPageIds[$channelPrefix . $pageId]->markaPageId;
                $finalPageInfos[$markaPageId] = $suggestedPageInfo;
            }
        }
    }

 }