<?php
/**
 * Gentics Portal.Node PHP
 * Author & Copyright (c) by Gentics Software GmbH
 * sales@gentics.com
 * http://www.gentics.com
 * Licenses can be found in the LICENSE.txt file in the root-folder of this installation
 * You must not use this software without a valid license agreement.
 *
 * Controller, which gets and renders suggestions from the Marketing Automation tracker.
 */
class SuggestionsController extends BaseController
{
    /**
     * The minimum number of suggestions that a client may request.
     * @var int
     */
    private $minSuggestionsCount = 1;
    
    /**
     * The maximum number of suggestions that a client may request.
     * @var int
     */
    private $maxSuggestionsCount = 100;
    
    /**
     * The timeout (in seconds) for requests to the Marketing Automation tracker.
     * @var int 
     */
    private $markaRequestTimeoutSec = 10;
    
    /**
     * An optional array of page attributes, which should be retrieved for each
     * suggested page and passed on to the view in addition to 'name' and 'url'.
     * Note that these attributes are CMS attributes like 'filename' or 
     * 'publishdate', not Marketing Automation attributes. For example:
     * array('folder_id', 'edittimestamp')
     * A value of NULL means that only 'name' and 'url' will be retrieved.
     * @var array 
     */
    private $additionalPageAttributes;
    
    /**
     * The prefix for cache entries (suggested page infos).
     * To make sure that suggested page infos with different page attributes
     * (when different additionalPageAttributes were used) are cached 
     * independently, implode($additionalPageAttributes) . '_' is appended to this prefix.
     * @var string 
     */
    private $cachePrefix = 'Marka_';

    /**
     * The Marketing Automation module.
     */
    private $markaModule;
    
    /**
     * Retrieves suggestions for the current user from the Marketing Automation tracker
     * and renders them using the specified view and sends them as the response
     * to the request. If no suggestions are returned by Marketing Automation, 
     * the response may be empty or may contain some default links (depending on the view),
     * but the status code will still be 200, if an error occurs, the 
     * appropriate error status code will be sent.
     * 
     * @param int $count The number of suggestions that should be retrieved.
     * 
     * @param string $view The view that should be used to render the suggestions.
     * This view is passed the parameter suggestedPages, which is an array
     * of the suggested pages infos in the order they were provided by
     * Marketing Automation (sorted by descending view count). Each of these
     * objects is an associative array that contains the requested page attributes
     * (e.g. array('name' => 'Soccer', 'url' => '/sports/Soccer.html').
     * If any of the attributes was not found for a page or was empty, the
     * entry in the array will exist, but will be NULL. If there are no
     * suggested pages, the parameter suggestedPages will be NULL and it is up 
     * to the view to either not render anything or to render some default links.
     * 
     * @param string $categories The list of categories that all returned suggestions must have 
     * (they may also have other categories as well).
     * This is optional and if set, it must be a string of the following format:
     * Category1;Category2;Category3;...
     * 
     * @param string $currentPageId The ID of the page that the user is currently 
     * viewing (if set, this will be excluded from the suggestions). See the
     * documentation in the MarkaSuggestionsWidget for more information about
     * this parameter.
     * 
     * @return void
     */
    public function actionSuggestions($count, $view, $categories = NULL, $currentPageId = NULL)
    {
        if (!is_numeric($count) || empty($view)) {
            header('HTTP/1.0 400 Bad Request' , true, 400);
            echo('One or both of the following query parameters was empty or invalid: count and view.');
            return;
        }
        if ($count < $this->minSuggestionsCount || $count > $this->maxSuggestionsCount) {
            header('HTTP/1.0 400 Bad Request', true, 400);
            echo('The count parameter must be between ' . strval($this->minSuggestionsCount) . ' and ' . strval($this->maxSuggestionsCount) . '.');
            return;
        }
        
        $this->markaModule = Yii::app()->getModule('marketingautomation');
        $this->additionalPageAttributes = $this->markaModule->suggestionsAdditionalPageAttributes;
        if (!empty($this->additionalPageAttributes)) {
            $this->cachePrefix .= implode($this->additionalPageAttributes) . '_';
        }
        $url = $this->markaModule->getMarkaTrackerBaseUrl() . '/v1/suggestions/';
        $url .= rawurlencode($this->markaModule->getUUID()) . '/' . strval($count);
        if (!empty($categories)) {
            $url .= '/' . self::_encodeCategories($categories);
        }
        if (!empty($currentPageId)) {
            $url .= '?currentPageId=' . rawurlencode($currentPageId);
        }
        $context = stream_context_create(array('http' => array('timeout' => $this->markaRequestTimeoutSec)));
        $responseJson = @file_get_contents($url, false, $context);
        $suggestedPages = NULL;
        if ($responseJson !== false) {
            $responseObj = json_decode($responseJson);
            if (!empty($responseObj->suggestions)) {
                $suggestedPages = $this->_getSuggestedPagesInfos($responseObj->suggestions);
                if (empty($suggestedPages)) {
                    $suggestedPages = NULL;
                }
            }
            $this->renderPartial($view, array('suggestedPages' => $suggestedPages));
        } else {
            header('HTTP/1.0 500 Internal Error', true, 500);
        }
    }
    
    /**
     * Retrieves the requested attributes from the content repository for
     * the specified Marketing Automation page IDs and returns an array
     * with all page infos found in the repository in the order that the page
     * IDs appear in the markaPageIds array. Each of these
     * objects is an associative array that contains the requested page attributes
     * (e.g. array('name' => 'Soccer', 'url' => '/sports/Soccer.html').
     * If any of the attributes was not found for a page or was empty, the
     * entry in the array will exist, but will be NULL.
     * 
     * @param array $markaPageIds the array of page IDs returned by Marketing Automation
     * @return array
     */
    private function _getSuggestedPagesInfos(&$markaPageIds)
    {
        $cachedPageInfos = $this->_checkForCachedPageInfos($markaPageIds);
        $requiredPageAttributes = $this->_createPageAttributesArray();
        $pageInfosProviders = $this->markaModule->getPageInfosProviders();
        $pageIdsGroupedByPrefix = self::_groupPageIdsByPrefix($markaPageIds, $pageInfosProviders, $cachedPageInfos);

        // Load the page infos using the providers.
        $newPageInfos = array();
        foreach ($pageIdsGroupedByPrefix as $prefix => $pageIds) {
            if (!empty($pageIds)) {
                $pageInfosForPrefix = $pageInfosProviders[$prefix]->getPageInfos($pageIds, $requiredPageAttributes);
                self::_array_merge_into($newPageInfos, $pageInfosForPrefix);
            }
        }

        // Add the newly loaded page infos to the cache.
        foreach ($newPageInfos as $pageId => $pageInfo) {
            $cacheKey = $this->_getCacheKey($pageId);
            if (Yii::app()->cache->set($cacheKey, $pageInfo, Yii::app()->getModule('marketingautomation')->cacheTime) === false) {
                Yii::log('MarketingAutomation SuggestionsController: Error adding page info to the cache.', CLogger::LEVEL_ERROR, 'php');
            }
        }

        return self::_createSuggestedPagesInfosArray($markaPageIds, $cachedPageInfos, $newPageInfos);
    }
    
    /**
     * Checks if any of the page IDs in the specified markaPageIds array
     * already have page infos in the cache and if so, adds a markaPageId => pageInfo
     * entry for each cached page info to the returned associative array. 
     * pageInfo is an associative array, which maps a page's attributes to their values,
     * (e.g. array('name' => 'Soccer', 'url' => '/sports/Soccer.html').
     * If no page IDs are found in the cache, the return value will be an empty array.
     * 
     * @param array $markaPageIds the array of Marketing Automation page IDs
     * @return array
     */
    private function _checkForCachedPageInfos($markaPageIds) {
        $ret = array();
        foreach ($markaPageIds as $pageId) {
            $cacheKey = $this->_getCacheKey($pageId);
            $cachedPageInfo = Yii::app()->cache->get($cacheKey);
            if ($cachedPageInfo !== false) {
                $ret[$pageId] = $cachedPageInfo;
            }
        }
        return $ret;
    }
    
    /**
     * Creates and returns an array, which contains all the page attributes that
     * should be retrieved from the content repository.
     * 
     * @return string
     */
    private function _createPageAttributesArray() {
        $baseAttributes = array('name', 'url');
        if (!empty($this->additionalPageAttributes)) {
            return array_merge($baseAttributes, $this->additionalPageAttributes);
        } else {
            return $baseAttributes;
        }
    }

    /**
     * Groups the specified markaPageIds by the prefixes specified as keys in the pageInfosProviders array and
     * returns an associative array, where the keys are the prefixes and the values are arrays of marka page IDs,
     * which have that prefix.
     *
     * @param array $markaPageIds the array of Marketing Automation page IDs
     * @param array $pageInfosProviders the associative array of page infos providers, whose keys (prefixes) will be used
     * for grouping the page IDs
     * @param array $cachedPageInfos the associative array of cached page infos. page IDs found as keys in this array,
     * will not be added to the returned array
     * @return array
     */
    private static function _groupPageIdsByPrefix($markaPageIds, $pageInfosProviders, $cachedPageInfos)
    {
        $pageIdsGroupedByPrefix = array('' => array());
        foreach ($markaPageIds as $pageId) {
            if (array_key_exists($pageId, $cachedPageInfos)) {
                continue;
            }
            $pageIdAdded = FALSE;
            foreach ($pageInfosProviders as $prefix => $provider) {
                if ($prefix !== '') {
                    if (strpos($pageId, $prefix) === 0) {
                        if (!array_key_exists($prefix, $pageIdsGroupedByPrefix)) {
                            $pageIdsGroupedByPrefix[$prefix] = array();
                        }
                        array_push($pageIdsGroupedByPrefix[$prefix], $pageId);
                        $pageIdAdded = TRUE;
                        break;
                    }
                }
            }
            if (!$pageIdAdded) {
                array_push($pageIdsGroupedByPrefix[''], $pageId);
            }
        }
        return $pageIdsGroupedByPrefix;
    }

    /**
     * Merges the entries from the associative array src into the 
     * associative array dest.
     *
     * @param array $dest the associative array, to which the entries from src should be added
     * @param array $src the associative array, whose entries should be added to dest.
     */
    private static function _array_merge_into(&$dest, $src)
    {
        foreach ($src as $key => $value) {
            $dest[$key] = $value;
        }
    }
    
    /**
     * Creates the final suggested pages infos array, which has the same order
     * as the page IDs, which were retrieved from the Marketing Automation system.
     * Pages, for which no page infos could be found, are not included in the
     * returned array.
     * 
     * @param array $markaPageIds the array of MarkA page IDs, in the order they were supplied by the
     * Marketing Automation system
     * @param array $cachedPageInfos the associative array of page infos that were found in the cache,
     * each entry must be pageId => pageInfo
     * @param array $newPageInfos the associative array of page infos that were loaded using a provider,
     * each entry must be pageId => pageInfo
     * @return array
     */
    private static function _createSuggestedPagesInfosArray($markaPageIds, $cachedPageInfos, $newPageInfos) {
        $suggestedPagesInfos = array();
        $pageIdsLength = count($markaPageIds);
        for ($i = 0; $i < $pageIdsLength; ++$i) {
            $pageId = $markaPageIds[$i];
            if (array_key_exists($pageId, $cachedPageInfos)) {
                array_push($suggestedPagesInfos, $cachedPageInfos[$pageId]);
            } else {
                if (array_key_exists($pageId, $newPageInfos)) {
                    array_push($suggestedPagesInfos, $newPageInfos[$pageId]);
                }
            }
        }
        return $suggestedPagesInfos;
    }
    
    /**
     * Gets the cache key for the specified Marketing Automation Page ID.
     * 
     * @param string $markaPageId the Marketing Automation Page ID, for which to get the cache key
     * @return string
     */
    private function _getCacheKey($markaPageId)
    {
        return $this->cachePrefix . $markaPageId;
    }
    
    /**
     * Converts the specified categories array into a categories string for
     * the get suggestions REST request and returns that string. This string
     * is of the form category1;category2;category3;category4
     * @param array $categories
     * @return string
     */
    private static function _assembleCategoriesStr(&$categories)
    {
        $ret = '';
        $first = true;
        foreach ($categories as $category) {
            if (!$first) {
                $ret .= ';';
            } else {
                $first = false;
            }
            $ret .= rawurlencode($category);
        }
        return $ret;
    }
    
    /**
     * Encodes the categories string received as a parameter for the actionSuggestions()
     * function for being included in the URL of the request to the 
     * Marketing Automation tracker. The input string should be of the form
     * Category1;Category2;Category3;...
     * The output string will be of the same form, but the categories
     * will be URL encoded.
     * @param string $categoriesStr
     * @return string
     */
    private static function _encodeCategories($categoriesStr) {
        $categoriesArray = explode(';', $categoriesStr);
        return self::_assembleCategoriesStr($categoriesArray);
    }
}
